@@ -22,7 +22,7 @@
  * @author Samuel Mimram
  */
 
-/* $Id: alsa_stubs.c 3730 2007-05-22 11:32:53Z metamorph68 $ */
+/* $Id: alsa_stubs.c 4670 2007-10-17 16:34:56Z metamorph68 $ */
 
 #include <string.h>
 #include <alsa/asoundlib.h>
@@ -294,6 +294,80 @@ CAMLprim value ocaml_snd_pcm_writei(value handle_, value sbuf, value ofs_, value
   CAMLreturn(Val_int(ret));
 }
 
+CAMLprim value ocaml_snd_pcm_readn(value handle_, value dbuf, value ofs_, value len_)
+{
+  CAMLparam4(handle_, dbuf, ofs_, len_);
+  int len = Int_val(len_);
+  int ofs = Int_val(ofs_);
+  int chans = Wosize_val(dbuf);
+  snd_pcm_t *handle = Pcm_handle_val(handle_);
+  char **buf;
+  int c;
+  int ret;
+
+  /* TODO: check the size of dbuf */
+  buf = malloc(chans * sizeof(char*));
+  for(c = 0; c < chans; c++)
+    buf[c] = malloc(2 * len);
+
+  caml_enter_blocking_section();
+  ret = snd_pcm_readn(handle, (void**)buf, len);
+  caml_leave_blocking_section();
+
+  for(c = 0; c < chans; c++)
+  {
+    memcpy(String_val(Field(dbuf, c)) + ofs, buf[c], 2 * len);
+    free(buf[c]);
+  }
+  free(buf);
+  if (ret == -EPIPE)
+    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
+  else if (ret == -EBADFD)
+    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
+  else if (ret == -ESTRPIPE)
+    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
+  check_for_err(ret);
+
+  CAMLreturn(Val_int(ret));
+}
+
+CAMLprim value ocaml_snd_pcm_writen(value handle_, value sbuf, value ofs_, value len_)
+{
+  CAMLparam4(handle_, sbuf, ofs_, len_);
+  int len = Int_val(len_);
+  int ofs = Int_val(ofs_);
+  int chans = Wosize_val(sbuf);
+  snd_pcm_t *handle = Pcm_handle_val(handle_);
+  char **buf;
+  int c;
+  int ret;
+
+  /* TODO: check the size of sbuf */
+  buf = malloc(chans * sizeof(char*));
+  for(c = 0; c < chans; c++)
+  {
+    buf[c] = malloc(2 * len);
+    memcpy(buf[c], String_val(Field(sbuf, c)) + ofs, 2 * len);
+  }
+
+  caml_enter_blocking_section();
+  ret = snd_pcm_writen(handle, (void**)buf, len);
+  caml_leave_blocking_section();
+
+  for(c = 0; c < chans; c++)
+    free(buf[c]);
+  free(buf);
+  if (ret == -EPIPE)
+    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
+  else if (ret == -EBADFD)
+    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
+  else if (ret == -ESTRPIPE)
+    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
+  check_for_err(ret);
+
+  CAMLreturn(Val_int(ret));
+}
+
 CAMLprim value ocaml_snd_pcm_readn_float(value handle_, value dbuf, value ofs_, value len_)
 {
   CAMLparam4(handle_, dbuf, ofs_, len_);
@@ -305,7 +379,7 @@ CAMLprim value ocaml_snd_pcm_readn_float(value handle_, value dbuf, value ofs_,
   int c, i;
   int ret;
 
-  /* TODO: check the size of fbuf */
+  /* TODO: check the size of dbuf */
   buf = malloc(chans * sizeof(float*));
   for(c = 0; c < chans; c++)
     buf[c] = malloc(len * sizeof(float));
@@ -671,6 +745,16 @@ CAMLprim value ocaml_snd_pcm_set_buffer_size_near(value handle, value params, va
   CAMLreturn(Val_int(s));
 }
 
+CAMLprim value ocaml_snd_pcm_get_buffer_size_min(value params)
+{
+  CAMLparam1(params);
+  snd_pcm_uframes_t ans;
+
+  check_for_err(snd_pcm_hw_params_get_buffer_size_min(Hw_params_val(params), &ans));
+
+  CAMLreturn(Val_int(ans));
+}
+
 static int get_access(value access)
 {
   int a = Int_val(access);
@@ -686,7 +770,11 @@ static int get_access(value access)
 
 CAMLprim value ocaml_snd_set_access(value handle, value params, value access)
 {
+  CAMLparam3(handle, params, access);
+
   check_for_err(snd_pcm_hw_params_set_access(Pcm_handle_val(handle), Hw_params_val(params), get_access(access)));
+
+  CAMLreturn(Val_unit);
 }
 
 CAMLprim value ocaml_snd_pcm_set_nonblock(value handle, value nonblocking)
